package net.locusworks.common.crypto;

import java.security.KeyFactory;
import java.security.KeyFactorySpi;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.PublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;

import sun.security.jca.GetInstance;
import sun.security.jca.GetInstance.Instance;

public class EncryptionKeyFactory extends KeyFactory {

  protected EncryptionKeyFactory(KeyFactorySpi keyFacSpi, Provider provider, String algorithm) {
    super(keyFacSpi, provider, algorithm);
  }
  
  public PrivateKey generatePrivateKey(KeySpec keySpec) throws InvalidKeySpecException {
    if (keySpec instanceof AESKeySpec) {
      return ((AESKeySpec)keySpec).generateKey();
    }
    return super.generatePrivate(keySpec);
  }
  
  public PublicKey generatePublicKey(KeySpec keySpec) throws InvalidKeySpecException {
    keySpec = keySpec instanceof SSHEncodedKeySpec ? ((SSHEncodedKeySpec)keySpec).convertToRSAPubKeySpec() : keySpec;
    return super.generatePublic(keySpec);
  }
  
  public static EncryptionKeyFactory getInstance(String algorithm) throws NoSuchAlgorithmException {
    Instance instance = GetInstance.getInstance("KeyFactory", KeyFactorySpi.class, algorithm);
    return new EncryptionKeyFactory((KeyFactorySpi)instance.impl, instance.provider, algorithm);
  }

}
