package net.locusworks.common.immutables;

/**
 * Class that holds three immutable objects as triplets
 * @author Isaac Parenteau
 * @version 1.0.0 
 * @param <V1> class type of object 1
 * @param <V2> class type of object 2
 * @param <V3> class type of object 3
 */
public class Triplet<V1, V2, V3> extends Pair<V1, V2> {
  
  private V3 value3;
  
  /**
   * default constructor with no values
   */
  public Triplet() {
    super();
  }
  
  /**
   * Constructor 
   * @param value1 Object 1
   * @param value2 Object 2
   * @param value3 Object 3
   */
  public Triplet(V1 value1, V2 value2, V3 value3) {
    super(value1, value2);
    this.value3 = value3;
  }
  
  /**
   * Set value 3
   * @param value3 value 3
   */
  public void setValue3(V3 value3) {
    this.value3 = value3;
  }
  
  /**
   * Get value 3
   * @return value 3
   */
  public V3 getValue3() {
    return value3;
  }
  
  @Override
  public boolean equals(Object other) {
    if (!(other instanceof Triplet)) return false;
    
    Triplet<?, ?, ?> otherTriplet = (Triplet<?, ?, ?>)other;
    
    return super.equals(otherTriplet) && this.getValue3().equals(otherTriplet.getValue3());
  }
}