package net.locusworks.common.utils;

import java.util.Arrays;
import java.util.Iterator;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

/**
 * Utility class to make iterators streamable
 * @author Isaac Parenteau
 * @version 1.0.0
 * @date 02/15/2018
 */
public class StreamUtils {
  
  /**
   * Convert a iterator to a stream
   * @param iterator the iterator to convert
   * @param <T> the class type
   * @return stream of the iterator
   */
  public static <T> Stream<T> asStream(Iterator<T> iterator) {
    return asStream(iterator, false);
  }
  
  public static <T> Stream<T> asStream(Iterable<T> iterable) {
    return asStream(iterable, false);
  }
  
  /**
   * Converts an array to a stream
   * @param items the items to convert
   * @param <T> the class type
   * @return stream of the array
   */
  public static <T> Stream<T> asStream(T[] items) {
    return asStream(Arrays.asList(items).iterator(), false);
  }
  
  /**
   * Converts an array to a stream
   * @param items the items to convert
   * @param parallel make the stream parallel if set to true
   * @param <T> the class type
   * @return stream of the array
   */
  public static <T> Stream<T> asStream(T[] items, boolean parallel) {
    return asStream(Arrays.asList(items).iterator(), parallel);
  }
  
  public static <T> Stream<T> asStream(Iterable<T> iterable, boolean parallel) {
    return StreamSupport.stream(iterable.spliterator(), parallel);
  }
  
  /**
   * Convert an iterator to a stream
   * @param iterator iterator to convert
   * @param parallel make the stream parallel if set to true.
   * @param <T> the class type 
   * @return stream of the iterator
   */
  public static <T> Stream<T> asStream(Iterator<T> iterator, boolean parallel) {
    Iterable<T> iterable = () -> iterator;
    return StreamSupport.stream(iterable.spliterator(), parallel);
  }
}
